#!/usr/bin/python
#  
#  vim: expandtab:tw=68:ts=4:sw=4:
# 
#  (c) Copyright 2008 Samsung Telecommunications America, Inc.
#                   All rights reserved
# 
#              Wireless Terminals Lab R & D
# 
# 
#  File: install_gtk.py
#  Author: mreddy
#  Creation date: Wed May 21 13:37:07 2008
#  Rev: $Id$
# 

import sys;
import os;
import getopt;
import string;
import urllib;

from config import Pkg_info;

def usage(prog_name) :
    fmt = string.Template ("""
$prog_name: A script to download, patch, build and install gtk and its dependencies automatically.

Usage: $prog_name [Options]

Options:
  --help        Show this help message and quit.
  --install-prefix=p    Path where the gtk libs should be installed
  --src-prefix=p    Path where the pre-downloaded gtk source directories exist
  --download        Download the packages from the Internet
  --patch-prefix=p  Path where the patches exist
  --clean           Execute "make distclean" before building the package

Example:

1) Download, patch, build and install packages (Generally for the first time)

   $prog_name --install-prefix=/usr/local/dfb --src-prefix=/root/packages --patch-prefix=/root/patches --download

2) Just build and install packages

   $prog_name --install-prefix=/usr/local/dfb --src-prefix=/root/packages

3) Clean build and install packages

   $prog_name --install-prefix=/usr/local/dfb --src-prefix=/root/packages --clean

""");

    prog_name = prog_name[prog_name.rfind ("/") + 1:];
    print fmt.substitute (dict (prog_name = prog_name));


def exec_shell_cmd (cmd) :
    print "Executing: " + cmd + "\n";

    if os.system (cmd) != 0 :
        print "Command " + cmd + " failed\n";
        sys.exit(3);


def make_abs_prefix (prefix) :
    if prefix[0] != "/" :
        prefix = os.getcwd () + "/" + prefix;

    return prefix;


def parse_args (prog_name, args, conf) :
    try :
        opts, args = getopt.getopt (args, "", ["help", "install-prefix=", "src-prefix=", "download", "patch-prefix=", "clean"]);
    except getopt.GetoptError:
        print "Invalid arguments passed. Try " +  prog_name + " --help";
        usage (prog_name);
        sys.exit(2);

    conf["install-prefix"] = "";
    conf["src-prefix"] = os.getcwd ();
    conf["download"] = 0;
    conf["patch-prefix"] = "";
    conf["clean"] = 0;

    for opt, arg in opts :
        if opt == "--help" :
            usage (prog_name);
            sys.exit(2);
        elif opt == "--install-prefix" :
            conf["install-prefix"] = make_abs_prefix (arg);
        elif opt == "--src-prefix" :
            conf["src-prefix"] = make_abs_prefix (arg);
        elif opt == "--download" :
            conf["download"] = 1;
        elif opt == "--patch-prefix" :
            conf["patch-prefix"] = make_abs_prefix (arg);
        elif opt == "--clean" :
            conf["clean"] = 1;

    if conf["install-prefix"] == "":
        print "--install-prefix option is necessary\n";
        sys.exit(2);


def dfb_env_setup (prefix) :
    old_dir = os.getcwd();
    print "install-prefix = " + prefix + "\n";
    if os.path.exists (prefix) == 0 :
        os.mkdir (prefix);
    os.chdir (prefix);
    exec_shell_cmd ('mkdir -p man/man1 bin lib etc include info sbin share src');
    os.chdir (old_dir);

    os.environ['LD_LIBRARY_PATH'] = prefix + "/lib";
    os.environ['PKG_CONFIG'] = prefix + "/bin/pkg-config";
    os.environ['PKG_CONFIG_PATH'] = prefix + "/lib/pkgconfig";
    os.environ['PATH'] = prefix + "/bin:" + os.environ['PATH'];
    os.environ['CPPFLAGS'] = "-I" + prefix + "/include";
    os.environ['LDFLAGS'] = "-L" + prefix + "/lib";

    #print os.environ['LD_LIBRARY_PATH'], os.environ['PKG_CONFIG'], os.environ['PKG_CONFIG_PATH'], os.environ['PATH'], os.environ['CPPFLAGS'], os.environ['LDFLAGS'];


def download_and_unpack_pkg (pkg, conf) :

    url = pkg["download"];
    file = url[url.rfind ("/") + 1:];
    print "Downloading " + url + " to " + file;
    urllib.urlretrieve (url, file);

    type = file[file.rfind (".") + 1:];

    if type == "bz2" :
        cmd = "tar -xjvf " + file;
    elif type == "gz" :
        cmd = "tar -xzvf " + file;

    exec_shell_cmd (cmd);

def patch_pkg (pkg, conf) :
    for patch in pkg["patches"] :
        patch_cmd = "patch -p1 < " + conf["patch-prefix"] + "/" + patch;
        exec_shell_cmd (patch_cmd);


def build_pkg (pkg, conf) :

    conf_opts = "";
    if pkg.has_key ("config-opts") :
        conf_opts = pkg["config-opts"];

    config_cmd = "./configure " + "--prefix=" + conf["install-prefix"] + " " + conf_opts;

    if conf["clean"] :
        exec_shell_cmd ("make distclean");

    exec_shell_cmd (config_cmd);
    exec_shell_cmd ("make");


def install_pkg (pkg) :
    exec_shell_cmd ("make install");


if __name__ == "__main__" :
    Conf = {};
    parse_args (sys.argv[0], sys.argv[1:], Conf);

    dfb_env_setup (Conf["install-prefix"]);

    for pkg in Pkg_info :
        os.chdir (Conf["src-prefix"]);

        if Conf["download"] == 1 :
            download_and_unpack_pkg (pkg, Conf);

        os.chdir (Conf["src-prefix"] + "/"+ pkg["name"]);

        if Conf["patch-prefix"] != "" and pkg.has_key ("patches") :
            patch_pkg (pkg, Conf);

        build_pkg (pkg, Conf);
        install_pkg (pkg);


# EOF

